@@ -58,7 +58,7 @@ typedef struct {
 	uint32_t              server_id;
 	bool                  advanced;
 	php_phongo_bson_state visitor_data;
-	int                   got_iterator;
+	bool                  got_iterator;
 	long                  current;
 	char*                 database;
 	char*                 collection;
