@@ -14,11 +14,12 @@
  * limitations under the License.
  */
 
+#include <php.h>
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <php.h>
 #include "phongo_compat.h"
 #include "php_phongo.h"
 #include "php_bson.h"
@@ -36,10 +37,6 @@ PHP_FUNCTION(MongoDB_BSON_fromPHP)
 	zval*   data;
 	bson_t* bson;
 
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(this_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used) /* We don't use these */
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "A", &data) == FAILURE) {
 		return;
 	}
@@ -60,10 +57,6 @@ PHP_FUNCTION(MongoDB_BSON_toPHP)
 	zval*                 typemap = NULL;
 	php_phongo_bson_state state   = PHONGO_BSON_STATE_INITIALIZER;
 
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(this_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used) /* We don't use these */
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|a!", &data, &data_len, &typemap) == FAILURE) {
 		return;
 	}
@@ -96,10 +89,6 @@ PHP_FUNCTION(MongoDB_BSON_fromJSON)
 	bson_t              bson  = BSON_INITIALIZER;
 	bson_error_t        error = { 0 };
 
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(this_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used) /* We don't use these */
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &json, &json_len) == FAILURE) {
 		return;
 	}
@@ -122,10 +111,6 @@ static void phongo_bson_to_json(INTERNAL_FUNCTION_PARAMETERS, php_phongo_json_mo
 	char*               json = NULL;
 	size_t              json_len;
 
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(this_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used) /* We don't use these */
-
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &data, &data_len) == FAILURE) {
 		return;
 	}
