@@ -14,14 +14,14 @@
  * limitations under the License.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <php.h>
 #include <Zend/zend_interfaces.h>
 #include <ext/spl/spl_iterators.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "php_array_api.h"
 #include "phongo_compat.h"
 #include "php_phongo.h"
@@ -73,6 +73,50 @@ static inline bool php_phongo_bulkwrite_update_has_operators(bson_t* bupdate) /*
 	return false;
 } /* }}} */
 
+/* Returns whether the update document is considered an aggregation pipeline */
+static inline bool php_phongo_bulkwrite_update_is_pipeline(bson_t* bupdate) /* {{{ */
+{
+	bson_iter_t iter;
+	bson_iter_t child;
+	const char* key;
+	int         i = 0;
+	char*       i_str;
+
+	if (!bson_iter_init(&iter, bupdate)) {
+		return false;
+	}
+
+	while (bson_iter_next(&iter)) {
+		key   = bson_iter_key(&iter);
+		i_str = bson_strdup_printf("%d", i++);
+
+		if (strcmp(key, i_str)) {
+			bson_free(i_str);
+			return false;
+		}
+
+		bson_free(i_str);
+
+		if (BSON_ITER_HOLDS_DOCUMENT(&iter)) {
+			if (!bson_iter_recurse(&iter, &child)) {
+				return false;
+			}
+			if (!bson_iter_next(&child)) {
+				return false;
+			}
+			key = bson_iter_key(&child);
+			if (key[0] != '$') {
+				return false;
+			}
+		} else {
+			return false;
+		}
+	}
+
+	/* should return false when the document is empty */
+	return i != 0;
+} /* }}} */
+
 /* Returns whether the BSON array's keys are a sequence of integer strings
  * starting with "0". BSON_APPEND_ARRAY considers it the caller's responsibility
  * to ensure that the array's keys are properly formatted. */
@@ -243,9 +287,6 @@ static PHP_METHOD(BulkWrite, __construct)
 	zend_error_handling     error_handling;
 	zval*                   options = NULL;
 	zend_bool               ordered = 1;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling TSRMLS_CC);
 	intern = Z_BULKWRITE_OBJ_P(getThis());
@@ -283,7 +324,6 @@ static PHP_METHOD(BulkWrite, insert)
 	int                     bson_flags = PHONGO_BSON_ADD_ID;
 	bson_error_t            error      = { 0 };
 	DECLARE_RETURN_VALUE_USED
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
@@ -326,9 +366,6 @@ static PHP_METHOD(BulkWrite, update)
 	zval *                  zquery, *zupdate, *zoptions = NULL;
 	bson_t                  bquery = BSON_INITIALIZER, bupdate = BSON_INITIALIZER, boptions = BSON_INITIALIZER;
 	bson_error_t            error = { 0 };
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
@@ -352,7 +389,7 @@ static PHP_METHOD(BulkWrite, update)
 		goto cleanup;
 	}
 
-	if (php_phongo_bulkwrite_update_has_operators(&bupdate)) {
+	if (php_phongo_bulkwrite_update_has_operators(&bupdate) || php_phongo_bulkwrite_update_is_pipeline(&bupdate)) {
 		if (zoptions && php_array_existsc(zoptions, "multi") && php_array_fetchc_bool(zoptions, "multi")) {
 			if (!mongoc_bulk_operation_update_many_with_opts(intern->bulk, &bquery, &bupdate, &boptions, &error)) {
 				phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
@@ -392,9 +429,6 @@ static PHP_METHOD(BulkWrite, delete)
 	zval *                  zquery, *zoptions = NULL;
 	bson_t                  bquery = BSON_INITIALIZER, boptions = BSON_INITIALIZER;
 	bson_error_t            error = { 0 };
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
@@ -436,9 +470,6 @@ cleanup:
 static PHP_METHOD(BulkWrite, count)
 {
 	php_phongo_bulkwrite_t* intern;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
