@@ -14,14 +14,14 @@
  * limitations under the License.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <php.h>
 #include <Zend/zend_interfaces.h>
 #include <ext/spl/spl_iterators.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "phongo_compat.h"
 #include "php_phongo.h"
 #include "php_bson.h"
@@ -126,11 +126,12 @@ static void php_phongo_cursor_iterator_move_forward(zend_object_iterator* iter T
 		php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &cursor->visitor_data);
 	} else {
 		bson_error_t error = { 0 };
+		const bson_t* doc = NULL;
 
-		if (mongoc_cursor_error(cursor->cursor, &error)) {
+		if (mongoc_cursor_error_document(cursor->cursor, &error, &doc)) {
 			/* Intentionally not destroying the cursor as it will happen
 			 * naturally now that there are no more results */
-			phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
+			phongo_throw_exception_from_bson_error_t_and_reply(&error, doc TSRMLS_CC);
 		}
 	}
 
@@ -193,7 +194,7 @@ static zend_object_iterator* php_phongo_cursor_get_iterator(zend_class_entry* ce
 		return NULL;
 	}
 
-	cursor->got_iterator = 1;
+	cursor->got_iterator = true;
 
 	cursor_it = ecalloc(1, sizeof(php_phongo_cursor_iterator));
 #if PHP_VERSION_ID >= 70000
@@ -224,9 +225,6 @@ static PHP_METHOD(Cursor, setTypeMap)
 	php_phongo_bson_state state                   = PHONGO_BSON_STATE_INITIALIZER;
 	zval*                 typemap                 = NULL;
 	bool                  restore_current_element = false;
-	SUPPRESS_UNUSED_WARNING(return_value)
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_CURSOR_OBJ_P(getThis());
 
@@ -297,9 +295,6 @@ static int php_phongo_cursor_to_array_apply(zend_object_iterator* iter, void* pu
    Returns an array of all result documents for this cursor */
 static PHP_METHOD(Cursor, toArray)
 {
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
-
 	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
@@ -317,8 +312,6 @@ static PHP_METHOD(Cursor, toArray)
 static PHP_METHOD(Cursor, getId)
 {
 	php_phongo_cursor_t* intern;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_CURSOR_OBJ_P(getThis());
 
@@ -334,8 +327,6 @@ static PHP_METHOD(Cursor, getId)
 static PHP_METHOD(Cursor, getServer)
 {
 	php_phongo_cursor_t* intern;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_CURSOR_OBJ_P(getThis());
 
@@ -351,8 +342,6 @@ static PHP_METHOD(Cursor, getServer)
 static PHP_METHOD(Cursor, isDead)
 {
 	php_phongo_cursor_t* intern;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_CURSOR_OBJ_P(getThis());
 
@@ -572,14 +561,14 @@ void php_phongo_cursor_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_cursor_ce);
 	php_phongo_cursor_ce->get_iterator = php_phongo_cursor_get_iterator;
 
+	zend_class_implements(php_phongo_cursor_ce TSRMLS_CC, 1, php_phongo_cursor_interface_ce);
+
 	memcpy(&php_phongo_handler_cursor, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_cursor.get_debug_info = php_phongo_cursor_get_debug_info;
 #if PHP_VERSION_ID >= 70000
 	php_phongo_handler_cursor.free_obj = php_phongo_cursor_free_object;
 	php_phongo_handler_cursor.offset   = XtOffsetOf(php_phongo_cursor_t, std);
 #endif
-
-	zend_class_implements(php_phongo_cursor_ce TSRMLS_CC, 1, zend_ce_traversable);
 } /* }}} */
 
 /*
