@@ -14,13 +14,13 @@
  * limitations under the License.
  */
 
+#include <php.h>
+#include <Zend/zend_interfaces.h>
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <php.h>
-#include <Zend/zend_interfaces.h>
-
 #include "phongo_compat.h"
 #include "php_phongo.h"
 
@@ -33,8 +33,6 @@ static PHP_METHOD(CursorId, __toString)
 	php_phongo_cursorid_t* intern;
 	char*                  tmp;
 	int                    tmp_len;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_CURSORID_OBJ_P(getThis());
 
