@@ -14,18 +14,60 @@
  * limitations under the License.
  */
 
+#include <php.h>
+#include <Zend/zend_interfaces.h>
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <php.h>
-#include <Zend/zend_interfaces.h>
-
 #include "phongo_compat.h"
 #include "php_phongo.h"
 
 zend_class_entry* php_phongo_readconcern_ce;
 
+/* Initialize the object from a HashTable and return whether it was successful.
+ * An exception will be thrown on error. */
+static bool php_phongo_readconcern_init_from_hash(php_phongo_readconcern_t* intern, HashTable* props TSRMLS_DC) /* {{{ */
+{
+#if PHP_VERSION_ID >= 70000
+	zval* level;
+
+	intern->read_concern = mongoc_read_concern_new();
+
+	if ((level = zend_hash_str_find(props, "level", sizeof("level") - 1))) {
+		if (Z_TYPE_P(level) == IS_STRING) {
+			mongoc_read_concern_set_level(intern->read_concern, Z_STRVAL_P(level));
+			return true;
+		}
+
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "%s initialization requires \"level\" string field", ZSTR_VAL(php_phongo_readconcern_ce->name));
+		goto failure;
+	}
+#else
+	zval** level;
+
+	intern->read_concern = mongoc_read_concern_new();
+
+	if (zend_hash_find(props, "level", sizeof("level"), (void**) &level) == SUCCESS) {
+		if (Z_TYPE_PP(level) == IS_STRING) {
+			mongoc_read_concern_set_level(intern->read_concern, Z_STRVAL_PP(level));
+			return true;
+		}
+
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "%s initialization requires \"level\" string field", ZSTR_VAL(php_phongo_readconcern_ce->name));
+		goto failure;
+	}
+#endif
+
+	return true;
+
+failure:
+	mongoc_read_concern_destroy(intern->read_concern);
+	intern->read_concern = NULL;
+	return false;
+} /* }}} */
+
 /* {{{ proto void MongoDB\Driver\ReadConcern::__construct([string $level])
    Constructs a new ReadConcern */
 static PHP_METHOD(ReadConcern, __construct)
@@ -35,10 +77,6 @@ static PHP_METHOD(ReadConcern, __construct)
 	char*                     level     = NULL;
 	phongo_zpp_char_len       level_len = 0;
 
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
-
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling TSRMLS_CC);
 	intern = Z_READCONCERN_OBJ_P(getThis());
 
@@ -55,14 +93,32 @@ static PHP_METHOD(ReadConcern, __construct)
 	}
 } /* }}} */
 
+/* {{{ proto void MongoDB\BSON\ReadConcern::__set_state(array $properties)
+*/
+static PHP_METHOD(ReadConcern, __set_state)
+{
+	php_phongo_readconcern_t* intern;
+	HashTable*                props;
+	zval*                     array;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a", &array) == FAILURE) {
+		RETURN_FALSE;
+	}
+
+	object_init_ex(return_value, php_phongo_readconcern_ce);
+
+	intern = Z_READCONCERN_OBJ_P(return_value);
+	props  = Z_ARRVAL_P(array);
+
+	php_phongo_readconcern_init_from_hash(intern, props TSRMLS_CC);
+} /* }}} */
+
 /* {{{ proto string|null MongoDB\Driver\ReadConcern::getLevel()
    Returns the ReadConcern "level" option */
 static PHP_METHOD(ReadConcern, getLevel)
 {
 	php_phongo_readconcern_t* intern;
 	const char*               level;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_READCONCERN_OBJ_P(getThis());
 
@@ -85,8 +141,6 @@ static PHP_METHOD(ReadConcern, getLevel)
 static PHP_METHOD(ReadConcern, isDefault)
 {
 	php_phongo_readconcern_t* intern;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_READCONCERN_OBJ_P(getThis());
 
@@ -148,12 +202,17 @@ ZEND_BEGIN_ARG_INFO_EX(ai_ReadConcern___construct, 0, 0, 0)
 	ZEND_ARG_INFO(0, level)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(ai_ReadConcern___set_state, 0, 0, 1)
+	ZEND_ARG_ARRAY_INFO(0, properties, 0)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(ai_ReadConcern_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_readconcern_me[] = {
 	/* clang-format off */
 	PHP_ME(ReadConcern, __construct, ai_ReadConcern___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(ReadConcern, __set_state, ai_ReadConcern___set_state, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
 	PHP_ME(ReadConcern, getLevel, ai_ReadConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(ReadConcern, isDefault, ai_ReadConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(ReadConcern, bsonSerialize, ai_ReadConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
@@ -183,7 +242,7 @@ static void php_phongo_readconcern_free_object(phongo_free_object_arg* object TS
 #if PHP_VERSION_ID < 70000
 	efree(intern);
 #endif
-} /* }}} */
+}
 
 static phongo_create_object_retval php_phongo_readconcern_create_object(zend_class_entry* class_type TSRMLS_DC) /* {{{ */
 {
