@@ -28,7 +28,9 @@
 #include <netinet/tcp.h>
 #include <arpa/nameser.h>
 #include <resolv.h>
+#define BSON_INSIDE
 #include <bson/bson-string.h>
+#undef BSON_INSIDE
 
 #endif
 #endif
@@ -39,6 +41,7 @@
 #include "mongoc/mongoc-database-private.h"
 #include "mongoc/mongoc-gridfs-private.h"
 #include "mongoc/mongoc-error.h"
+#include "mongoc/mongoc-error-private.h"
 #include "mongoc/mongoc-log.h"
 #include "mongoc/mongoc-queue-private.h"
 #include "mongoc/mongoc-socket.h"
@@ -102,22 +105,31 @@ _mongoc_client_killcursors_command (mongoc_cluster_t *cluster,
 typedef bool (*mongoc_rr_callback_t) (const char *service,
                                       PDNS_RECORD pdns,
                                       mongoc_uri_t *uri,
+                                      mongoc_rr_data_t *rr_data,
                                       bson_error_t *error);
 
 static bool
 srv_callback (const char *service,
               PDNS_RECORD pdns,
               mongoc_uri_t *uri,
+              mongoc_rr_data_t *rr_data,
               bson_error_t *error)
 {
-   return mongoc_uri_append_host (
+   if (rr_data && rr_data->hosts) {
+      _mongoc_host_list_remove_host (
+         &(rr_data->hosts), pdns->Data.SRV.pNameTarget, pdns->Data.SRV.wPort);
+   }
+
+   return mongoc_uri_upsert_host (
       uri, pdns->Data.SRV.pNameTarget, pdns->Data.SRV.wPort, error);
 }
 
+/* rr_data is unused, but here to match srv_callback signature */
 static bool
 txt_callback (const char *service,
               PDNS_RECORD pdns,
               mongoc_uri_t *uri,
+              mongoc_rr_data_t *rr_data,
               bson_error_t *error)
 {
    DWORD i;
@@ -163,6 +175,7 @@ static bool
 _mongoc_get_rr_dnsapi (const char *service,
                        mongoc_rr_type_t rr_type,
                        mongoc_uri_t *uri,
+                       mongoc_rr_data_t *rr_data,
                        bson_error_t *error)
 {
    const char *rr_type_name;
@@ -238,7 +251,13 @@ _mongoc_get_rr_dnsapi (const char *service,
             DNS_ERROR ("Multiple TXT records for \"%s\"", service);
          }
 
-         if (!callback (service, pdns, uri, error)) {
+         if (rr_data) {
+            if ((i == 0) || (pdns->dwTtl < rr_data->min_ttl)) {
+               rr_data->min_ttl = pdns->dwTtl;
+            }
+         }
+
+         if (!callback (service, pdns, uri, rr_data, error)) {
             callback_success = false;
             GOTO (done);
          }
@@ -249,6 +268,10 @@ _mongoc_get_rr_dnsapi (const char *service,
       pdns = pdns->pNext;
    } while (pdns);
 
+   if (rr_data) {
+      rr_data->count = i;
+   }
+
 done:
    if (pdns) {
       DnsRecordListFree (pdns, DnsFreeRecordList);
@@ -267,6 +290,7 @@ typedef bool (*mongoc_rr_callback_t) (const char *service,
                                       ns_msg *ns_answer,
                                       ns_rr *rr,
                                       mongoc_uri_t *uri,
+                                      mongoc_rr_data_t *rr_data,
                                       bson_error_t *error);
 
 static bool
@@ -274,6 +298,7 @@ srv_callback (const char *service,
               ns_msg *ns_answer,
               ns_rr *rr,
               mongoc_uri_t *uri,
+              mongoc_rr_data_t *rr_data,
               bson_error_t *error)
 {
    const uint8_t *data;
@@ -300,17 +325,22 @@ srv_callback (const char *service,
                  strerror (h_errno));
    }
 
-   ret = mongoc_uri_append_host (uri, name, port, error);
+   if (rr_data && rr_data->hosts) {
+      _mongoc_host_list_remove_host (&(rr_data->hosts), name, port);
+   }
+   ret = mongoc_uri_upsert_host (uri, name, port, error);
 
 done:
    return ret;
 }
 
+/* rr_data is unused, but here to match srv_callback signature */
 static bool
 txt_callback (const char *service,
               ns_msg *ns_answer,
               ns_rr *rr,
               mongoc_uri_t *uri,
+              mongoc_rr_data_t *rr_data,
               bson_error_t *error)
 {
    char s[256];
@@ -372,6 +402,7 @@ static bool
 _mongoc_get_rr_search (const char *service,
                        mongoc_rr_type_t rr_type,
                        mongoc_uri_t *uri,
+                       mongoc_rr_data_t *rr_data,
                        bson_error_t *error)
 {
 #ifdef MONGOC_HAVE_RES_NSEARCH
@@ -430,6 +461,10 @@ _mongoc_get_rr_search (const char *service,
       DNS_ERROR ("No %s records for \"%s\"", rr_type_name, service);
    }
 
+   if (rr_data) {
+      rr_data->count = n;
+   }
+
    for (i = 0; i < n; i++) {
       if (i > 0 && rr_type == MONGOC_RR_TXT) {
          /* Initial DNS Seedlist Discovery Spec: a client "MUST raise an error
@@ -446,7 +481,17 @@ _mongoc_get_rr_search (const char *service,
                     strerror (h_errno));
       }
 
-      if (!callback (service, &ns_answer, &resource_record, uri, error)) {
+      if (rr_data) {
+         uint32_t ttl;
+
+         ttl = ns_rr_ttl (resource_record);
+         if ((i == 0) || (ttl < rr_data->min_ttl)) {
+            rr_data->min_ttl = ttl;
+         }
+      }
+
+      if (!callback (
+             service, &ns_answer, &resource_record, uri, rr_data, error)) {
          callback_success = false;
          GOTO (done);
       }
@@ -488,12 +533,13 @@ bool
 _mongoc_client_get_rr (const char *service,
                        mongoc_rr_type_t rr_type,
                        mongoc_uri_t *uri,
+                       mongoc_rr_data_t *rr_data,
                        bson_error_t *error)
 {
 #ifdef MONGOC_HAVE_DNSAPI
-   return _mongoc_get_rr_dnsapi (service, rr_type, uri, error);
+   return _mongoc_get_rr_dnsapi (service, rr_type, uri, rr_data, error);
 #elif (defined(MONGOC_HAVE_RES_NSEARCH) || defined(MONGOC_HAVE_RES_SEARCH))
-   return _mongoc_get_rr_search (service, rr_type, uri, error);
+   return _mongoc_get_rr_search (service, rr_type, uri, rr_data, error);
 #else
    bson_set_error (error,
                    MONGOC_ERROR_STREAM,
@@ -717,7 +763,7 @@ mongoc_client_default_stream_initiator (const mongoc_uri_t *uri,
    BSON_ASSERT (host);
 
 #ifndef MONGOC_ENABLE_SSL
-   if (mongoc_uri_get_ssl (uri)) {
+   if (mongoc_uri_get_tls (uri)) {
       bson_set_error (error,
                       MONGOC_ERROR_CLIENT,
                       MONGOC_ERROR_CLIENT_NO_ACCEPTABLE_PEER,
@@ -999,7 +1045,7 @@ _mongoc_client_new_from_uri (mongoc_topology_t *topology)
    BSON_ASSERT (topology);
 
 #ifndef MONGOC_ENABLE_SSL
-   if (mongoc_uri_get_ssl (topology->uri)) {
+   if (mongoc_uri_get_tls (topology->uri)) {
       MONGOC_ERROR ("Can't create SSL client, SSL not enabled in this build.");
       return NULL;
    }
@@ -1035,7 +1081,7 @@ _mongoc_client_new_from_uri (mongoc_topology_t *topology)
 
 #ifdef MONGOC_ENABLE_SSL
    client->use_ssl = false;
-   if (mongoc_uri_get_ssl (client->uri)) {
+   if (mongoc_uri_get_tls (client->uri)) {
       mongoc_ssl_opt_t ssl_opt = {0};
 
       _mongoc_ssl_opts_from_uri (&ssl_opt, client->uri);
@@ -1557,6 +1603,11 @@ retry:
    ret = mongoc_cluster_run_command_monitored (
       &client->cluster, &parts->assembled, reply, error);
 
+   if (is_retryable) {
+      _mongoc_write_error_update_if_unsupported_storage_engine (
+         ret, error, reply);
+   }
+
    /* If a retryable error is encountered and the write is retryable, select
     * a new writable stream and retry. If server selection fails or the selected
     * server does not support retryable writes, fall through and allow the
@@ -1593,9 +1644,75 @@ retry:
 }
 
 
+static bool
+_mongoc_client_retryable_read_command_with_stream (
+   mongoc_client_t *client,
+   mongoc_cmd_parts_t *parts,
+   mongoc_server_stream_t *server_stream,
+   bson_t *reply,
+   bson_error_t *error)
+{
+   mongoc_server_stream_t *retry_server_stream = NULL;
+   bool is_retryable = true;
+   bool ret;
+   bson_t reply_local;
+
+   if (reply == NULL) {
+      reply = &reply_local;
+   }
+
+   ENTRY;
+
+   BSON_ASSERT (parts->is_retryable_read);
+
+retry:
+   ret = mongoc_cluster_run_command_monitored (
+      &client->cluster, &parts->assembled, reply, error);
+
+   /* If a retryable error is encountered and the read is retryable, select
+    * a new readable stream and retry. If server selection fails or the selected
+    * server does not support retryable reads, fall through and allow the
+    * original error to be reported. */
+   if (is_retryable &&
+       _mongoc_read_error_get_type (ret, error, reply) ==
+          MONGOC_READ_ERR_RETRY) {
+      bson_error_t ignored_error;
+
+      /* each read command may be retried at most once */
+      is_retryable = false;
+
+      if (retry_server_stream) {
+         mongoc_server_stream_cleanup (retry_server_stream);
+      }
+
+      retry_server_stream =
+         mongoc_cluster_stream_for_reads (&client->cluster,
+                                          parts->read_prefs,
+                                          parts->assembled.session,
+                                          NULL,
+                                          &ignored_error);
+
+      if (retry_server_stream &&
+          retry_server_stream->sd->max_wire_version >=
+             WIRE_VERSION_RETRY_READS) {
+         parts->assembled.server_stream = retry_server_stream;
+         bson_destroy (reply);
+         GOTO (retry);
+      }
+   }
+
+   if (retry_server_stream) {
+      mongoc_server_stream_cleanup (retry_server_stream);
+   }
+
+   RETURN (ret);
+}
+
+
 static bool
 _mongoc_client_command_with_stream (mongoc_client_t *client,
                                     mongoc_cmd_parts_t *parts,
+                                    const mongoc_read_prefs_t *read_prefs,
                                     mongoc_server_stream_t *server_stream,
                                     bson_t *reply,
                                     bson_error_t *error)
@@ -1613,6 +1730,11 @@ _mongoc_client_command_with_stream (mongoc_client_t *client,
          client, parts, server_stream, reply, error));
    }
 
+   if (parts->is_retryable_read) {
+      RETURN (_mongoc_client_retryable_read_command_with_stream (
+         client, parts, server_stream, reply, error));
+   }
+
    RETURN (mongoc_cluster_run_command_monitored (
       &client->cluster, &parts->assembled, reply, error));
 }
@@ -1656,7 +1778,7 @@ mongoc_client_command_simple (mongoc_client_t *client,
 
    if (server_stream) {
       ret = _mongoc_client_command_with_stream (
-         client, &parts, server_stream, reply, error);
+         client, &parts, read_prefs, server_stream, reply, error);
    } else {
       /* reply initialized by mongoc_cluster_stream_for_reads */
       ret = false;
@@ -1865,7 +1987,7 @@ _mongoc_client_command_with_opts (mongoc_client_t *client,
    }
 
    ret = _mongoc_client_command_with_stream (
-      client, &parts, server_stream, reply_ptr, error);
+      client, &parts, user_prefs, server_stream, reply_ptr, error);
 
    reply_initialized = true;
 
@@ -2022,7 +2144,7 @@ mongoc_client_command_simple_with_server_id (
       parts.read_prefs = read_prefs;
 
       ret = _mongoc_client_command_with_stream (
-         client, &parts, server_stream, reply, error);
+         client, &parts, read_prefs, server_stream, reply, error);
 
       mongoc_cmd_parts_cleanup (&parts);
       mongoc_server_stream_cleanup (server_stream);
@@ -2688,7 +2810,9 @@ _mongoc_client_pop_server_session (mongoc_client_t *client, bson_error_t *error)
  *       command.
  *
  * Returns:
- *       True on success, false on error and @error is set.
+ *       True on success, false on error and @error is set. Will return false
+ *       if the session is from an outdated client generation, a holdover
+ *       from before a call to mongoc_client_reset.
  *
  * Side effects:
  *       None.
@@ -2739,7 +2863,7 @@ _mongoc_client_push_server_session (mongoc_client_t *client,
  *       End all server sessions in the topology's server session pool.
  *       Don't block long: if server selection or connecting fails, quit.
  *
- *       The server session pool becomes invalid, but it's *not* cleared.
+ *       The server session pool becomes invalid, but may not be empty.
  *       Destroy the topology after this without using any sessions.
  *
  *--------------------------------------------------------------------------
@@ -2807,6 +2931,29 @@ _mongoc_client_end_sessions (mongoc_client_t *client)
    }
 }
 
+void
+mongoc_client_reset (mongoc_client_t *client)
+{
+   BSON_ASSERT (client);
+
+   client->generation++;
+
+   /* Client sessions are owned and destroyed by the user, but we keep
+      local pointers to them for reference. On reset, clear our local
+      set without destroying the sessions or calling endSessions.
+      client_sessions has no dtor, so it won't destroy its items.
+
+      Destroying the local cache of client sessions here ensures they
+      cannot be used by future operations--lookup for them will fail. */
+   mongoc_set_destroy (client->client_sessions);
+   client->client_sessions = mongoc_set_new (8, NULL, NULL);
+
+   /* Server sessions are owned by us, so we clear the pool on reset. */
+   _mongoc_topology_clear_session_pool (client->topology);
+
+   mongoc_cluster_disconnect (&(client->cluster));
+}
+
 mongoc_change_stream_t *
 mongoc_client_watch (mongoc_client_t *client,
                      const bson_t *pipeline,
