@@ -251,9 +251,7 @@ mongoc_cluster_run_command_opquery (mongoc_cluster_t *cluster,
    if (compressor_id != -1 && IS_NOT_COMMAND ("ismaster") &&
        IS_NOT_COMMAND ("saslstart") && IS_NOT_COMMAND ("saslcontinue") &&
        IS_NOT_COMMAND ("getnonce") && IS_NOT_COMMAND ("authenticate") &&
-       IS_NOT_COMMAND ("createuser") && IS_NOT_COMMAND ("updateuser") &&
-       IS_NOT_COMMAND ("copydbsaslstart") &&
-       IS_NOT_COMMAND ("copydbgetnonce") && IS_NOT_COMMAND ("copydb")) {
+       IS_NOT_COMMAND ("createuser") && IS_NOT_COMMAND ("updateuser")) {
       output = _mongoc_rpc_compress (cluster, compressor_id, &rpc, error);
       if (output == NULL) {
          GOTO (done);
@@ -408,6 +406,7 @@ done:
 typedef enum {
    MONGOC_REPLY_ERR_TYPE_NONE,
    MONGOC_REPLY_ERR_TYPE_NOT_MASTER,
+   MONGOC_REPLY_ERR_TYPE_SHUTDOWN,
    MONGOC_REPLY_ERR_TYPE_NODE_IS_RECOVERING
 } reply_error_type_t;
 
@@ -436,10 +435,11 @@ _check_not_master_or_recovering_error (const mongoc_client_t *client,
 
    switch (error->code) {
    case 11600: /* InterruptedAtShutdown */
+   case 91:    /* ShutdownInProgress */
+      return MONGOC_REPLY_ERR_TYPE_SHUTDOWN;
    case 11602: /* InterruptedDueToReplStateChange */
    case 13436: /* NotMasterOrSecondary */
    case 189:   /* PrimarySteppedDown */
-   case 91:    /* ShutdownInProgress */
       return MONGOC_REPLY_ERR_TYPE_NODE_IS_RECOVERING;
    case 10107: /* NotMaster */
    case 13435: /* NotMasterNoSlaveOk */
@@ -461,6 +461,7 @@ handle_not_master_error (mongoc_cluster_t *cluster,
                          const bson_t *reply)
 {
    mongoc_topology_t *topology = cluster->client->topology;
+   mongoc_server_description_t *sd;
    bson_error_t error;
    reply_error_type_t error_type =
       _check_not_master_or_recovering_error (cluster->client, reply, &error);
@@ -469,8 +470,19 @@ handle_not_master_error (mongoc_cluster_t *cluster,
       /* Server Discovery and Monitoring Spec: "When the client sees a 'not
        * master' or 'node is recovering' error it MUST replace the server's
        * description with a default ServerDescription of type Unknown."
-       */
+       *
+       * The client MUST clear its connection pool for the server
+       * if the server is 4.0 or earlier, and MUST NOT clear its connection
+       * pool for the server if the server is 4.2 or later. */
+      sd = mongoc_topology_server_by_id (topology, server_id, &error);
+      if (sd->max_wire_version <= WIRE_VERSION_4_0 ||
+          error_type == MONGOC_REPLY_ERR_TYPE_SHUTDOWN) {
+         mongoc_cluster_disconnect_node (cluster, server_id, false, NULL);
+      }
+      mongoc_server_description_destroy (sd);
+
       mongoc_topology_invalidate_server (topology, server_id, &error);
+
       if (topology->single_threaded) {
          /* SDAM Spec: "For single-threaded clients, in the case of a 'not
           * master' error, the client MUST check the server immediately... For a
@@ -492,6 +504,14 @@ handle_not_master_error (mongoc_cluster_t *cluster,
    }
 }
 
+bool
+_in_sharded_txn (const mongoc_client_session_t *session)
+{
+   return session && _mongoc_client_session_in_txn_or_ending (session) &&
+          _mongoc_topology_get_type (session->client->topology) ==
+             MONGOC_TOPOLOGY_SHARDED;
+}
+
 /*
  *--------------------------------------------------------------------------
  *
@@ -528,6 +548,7 @@ mongoc_cluster_run_command_monitored (mongoc_cluster_t *cluster,
    bson_t reply_local;
    bson_error_t error_local;
    int32_t compressor_id;
+   bson_iter_t iter;
 
    server_stream = cmd->server_stream;
    server_id = server_stream->sd->id;
@@ -598,6 +619,19 @@ mongoc_cluster_run_command_monitored (mongoc_cluster_t *cluster,
 
    handle_not_master_error (cluster, server_id, reply);
 
+   if (retval && _in_sharded_txn (cmd->session) &&
+       bson_iter_init_find (&iter, reply, "recoveryToken")) {
+      bson_destroy (cmd->session->recovery_token);
+      if (BSON_ITER_HOLDS_DOCUMENT (&iter)) {
+         cmd->session->recovery_token =
+            bson_new_from_data (bson_iter_value (&iter)->value.v_doc.data,
+                                bson_iter_value (&iter)->value.v_doc.data_len);
+      } else {
+         MONGOC_ERROR ("Malformed recovery token from server");
+         cmd->session->recovery_token = NULL;
+      }
+   }
+
    if (reply == &reply_local) {
       bson_destroy (&reply_local);
    }
@@ -1493,6 +1527,48 @@ _mongoc_cluster_auth_node (
    RETURN (ret);
 }
 
+static bool
+_mongoc_cluster_disconnect_node_in_set (uint32_t id, void *item, void *ctx)
+{
+   mongoc_cluster_t *cluster = (mongoc_cluster_t *) ctx;
+
+   mongoc_cluster_disconnect_node (cluster, id, false, NULL);
+
+   return true;
+}
+
+/*
+ *--------------------------------------------------------------------------
+ *
+ * mongoc_cluster_disconnect --
+ *
+ *       Disconnects all nodes in this cluster.
+ *
+ * Returns:
+ *       None.
+ *
+ * Side effects:
+ *       Clears the cluster's set of nodes and frees them if pooled.
+ *
+ *--------------------------------------------------------------------------
+ */
+
+void
+mongoc_cluster_disconnect (mongoc_cluster_t *cluster)
+{
+   mongoc_topology_t *topology;
+
+   BSON_ASSERT (cluster);
+
+   topology = cluster->client->topology;
+   /* in the single-threaded use case we share topology's streams */
+   if (topology->single_threaded) {
+      mongoc_topology_scanner_disconnect (topology->scanner);
+   } else {
+      mongoc_set_for_each_with_id (
+         cluster->nodes, _mongoc_cluster_disconnect_node_in_set, cluster);
+   }
+}
 
 /*
  *--------------------------------------------------------------------------
@@ -1807,7 +1883,7 @@ mongoc_server_stream_t *
 mongoc_cluster_stream_for_server (mongoc_cluster_t *cluster,
                                   uint32_t server_id,
                                   bool reconnect_ok,
-                                  const mongoc_client_session_t *cs,
+                                  mongoc_client_session_t *cs,
                                   bson_t *reply,
                                   bson_error_t *error)
 {
@@ -1819,6 +1895,15 @@ mongoc_cluster_stream_for_server (mongoc_cluster_t *cluster,
    BSON_ASSERT (cluster);
    BSON_ASSERT (server_id);
 
+   if (cs && cs->server_id && cs->server_id != server_id) {
+      _mongoc_bson_init_if_set (reply);
+      bson_set_error (error,
+                      MONGOC_ERROR_COMMAND,
+                      MONGOC_ERROR_SERVER_SELECTION_INVALID_ID,
+                      "Requested server id does not matched pinned server id");
+      RETURN (NULL);
+   }
+
    if (!error) {
       error = &err_local;
    }
@@ -1831,6 +1916,17 @@ mongoc_cluster_stream_for_server (mongoc_cluster_t *cluster,
       mongoc_cluster_disconnect_node (cluster, server_id, true, error);
    }
 
+   if (_in_sharded_txn (cs)) {
+      _mongoc_client_session_pin (cs, server_id);
+   } else {
+      /* Transactions Spec: Additionally, any non-transaction operation using
+       * a pinned ClientSession MUST unpin the session and the operation MUST
+       * perform normal server selection. */
+      if (cs && !_mongoc_client_session_in_txn_or_ending (cs)) {
+         _mongoc_client_session_unpin (cs);
+      }
+   }
+
    RETURN (server_stream);
 }
 
@@ -2081,6 +2177,37 @@ mongoc_cluster_destroy (mongoc_cluster_t *cluster) /* INOUT */
    EXIT;
 }
 
+static uint32_t
+_mongoc_cluster_select_server_id (mongoc_client_session_t *cs,
+                                  mongoc_topology_t *topology,
+                                  mongoc_ss_optype_t optype,
+                                  const mongoc_read_prefs_t *read_prefs,
+                                  bson_error_t *error)
+{
+   uint32_t server_id;
+
+   if (_in_sharded_txn (cs)) {
+      server_id = cs->server_id;
+      if (!server_id) {
+         server_id = mongoc_topology_select_server_id (
+            topology, optype, read_prefs, error);
+         if (server_id) {
+            _mongoc_client_session_pin (cs, server_id);
+         }
+      }
+   } else {
+      server_id =
+         mongoc_topology_select_server_id (topology, optype, read_prefs, error);
+      /* Transactions Spec: Additionally, any non-transaction operation using a
+       * pinned ClientSession MUST unpin the session and the operation MUST
+       * perform normal server selection. */
+      if (cs && !_mongoc_client_session_in_txn_or_ending (cs)) {
+         _mongoc_client_session_unpin (cs);
+      }
+   }
+
+   return server_id;
+}
 
 /*
  *--------------------------------------------------------------------------
@@ -2104,7 +2231,7 @@ static mongoc_server_stream_t *
 _mongoc_cluster_stream_for_optype (mongoc_cluster_t *cluster,
                                    mongoc_ss_optype_t optype,
                                    const mongoc_read_prefs_t *read_prefs,
-                                   const mongoc_client_session_t *cs,
+                                   mongoc_client_session_t *cs,
                                    bson_t *reply,
                                    bson_error_t *error)
 {
@@ -2116,8 +2243,8 @@ _mongoc_cluster_stream_for_optype (mongoc_cluster_t *cluster,
 
    BSON_ASSERT (cluster);
 
-   server_id =
-      mongoc_topology_select_server_id (topology, optype, read_prefs, error);
+   server_id = _mongoc_cluster_select_server_id (
+      cs, topology, optype, read_prefs, error);
 
    if (!server_id) {
       _mongoc_bson_init_with_transient_txn_error (cs, reply);
@@ -2126,8 +2253,8 @@ _mongoc_cluster_stream_for_optype (mongoc_cluster_t *cluster,
 
    if (!mongoc_cluster_check_interval (cluster, server_id)) {
       /* Server Selection Spec: try once more */
-      server_id =
-         mongoc_topology_select_server_id (topology, optype, read_prefs, error);
+      server_id = _mongoc_cluster_select_server_id (
+         cs, topology, optype, read_prefs, error);
 
       if (!server_id) {
          _mongoc_bson_init_with_transient_txn_error (cs, reply);
@@ -2164,7 +2291,7 @@ _mongoc_cluster_stream_for_optype (mongoc_cluster_t *cluster,
 mongoc_server_stream_t *
 mongoc_cluster_stream_for_reads (mongoc_cluster_t *cluster,
                                  const mongoc_read_prefs_t *read_prefs,
-                                 const mongoc_client_session_t *cs,
+                                 mongoc_client_session_t *cs,
                                  bson_t *reply,
                                  bson_error_t *error)
 {
@@ -2198,7 +2325,7 @@ mongoc_cluster_stream_for_reads (mongoc_cluster_t *cluster,
 
 mongoc_server_stream_t *
 mongoc_cluster_stream_for_writes (mongoc_cluster_t *cluster,
-                                  const mongoc_client_session_t *cs,
+                                  mongoc_client_session_t *cs,
                                   bson_t *reply,
                                   bson_error_t *error)
 {
@@ -2666,12 +2793,9 @@ network_error_reply (bson_t *reply, mongoc_cmd_t *cmd)
 {
    bson_t labels;
 
-   if (!reply) {
-      return;
+   if (reply) {
+      bson_init (reply);
    }
-
-   bson_init (reply);
-
    /* Transactions Spec defines TransientTransactionError: "Any
     * network error or server selection error encountered running any
     * command besides commitTransaction in a transaction. In the case
@@ -2679,6 +2803,17 @@ network_error_reply (bson_t *reply, mongoc_cmd_t *cmd)
     * network errors or server selection errors where the client
     * receives no server reply, the client adds the label." */
    if (_mongoc_client_session_in_txn (cmd->session) && !cmd->is_txn_finish) {
+      /* Transaction Spec: "Drivers MUST unpin a ClientSession when a command
+       * within a transaction, including commitTransaction and abortTransaction,
+       * fails with a TransientTransactionError". If we're about to add
+       * a TransientTransactionError label due to a client side error then we
+       * unpin. If commitTransaction/abortTransation includes a label in the
+       * server reply, we unpin in _mongoc_client_session_handle_reply. */
+      cmd->session->server_id = 0;
+      if (!reply) {
+         return;
+      }
+
       BSON_APPEND_ARRAY_BEGIN (reply, "errorLabels", &labels);
       BSON_APPEND_UTF8 (&labels, "0", TRANSIENT_TXN_ERR);
       bson_append_array_end (reply, &labels);
