@@ -14,6 +14,8 @@
  * limitations under the License.
  */
 
+#include "mongoc/mongoc-prelude.h"
+
 
 /*
  * Internal struct to represent a command we will send to the server - command
@@ -25,10 +27,6 @@
 #ifndef MONGOC_CMD_PRIVATE_H
 #define MONGOC_CMD_PRIVATE_H
 
-#if !defined(MONGOC_COMPILATION)
-#error "Only <mongoc/mongoc.h> can be included directly."
-#endif
-
 #include <bson/bson.h>
 
 #include "mongoc/mongoc-server-stream-private.h"
@@ -38,6 +36,14 @@
 
 BSON_BEGIN_DECLS
 
+#define MONGOC_DEFAULT_RETRYREADS true
+/* retryWrites requires sessions, which require crypto */
+#ifdef MONGOC_ENABLE_CRYPTO
+#define MONGOC_DEFAULT_RETRYWRITES true
+#else
+#define MONGOC_DEFAULT_RETRYWRITES false
+#endif
+
 typedef enum {
    MONGOC_CMD_PARTS_ALLOW_TXN_NUMBER_UNKNOWN,
    MONGOC_CMD_PARTS_ALLOW_TXN_NUMBER_YES,
@@ -73,6 +79,7 @@ typedef struct _mongoc_cmd_parts_t {
    bool is_write_command;
    bool prohibit_lsid;
    mongoc_cmd_parts_allow_txn_number_t allow_txn_number;
+   bool is_retryable_read;
    bool is_retryable_write;
    bool has_temp_session;
    mongoc_client_t *client;
@@ -125,6 +132,10 @@ mongoc_cmd_is_compressible (mongoc_cmd_t *cmd);
 void
 mongoc_cmd_parts_cleanup (mongoc_cmd_parts_t *op);
 
+bool
+_is_retryable_read (const mongoc_cmd_parts_t *parts,
+                    const mongoc_server_stream_t *server_stream);
+
 BSON_END_DECLS
 
 
