@@ -21,13 +21,14 @@
 #include "mongoc/mongoc-client-session-private.h"
 #include "mongoc/mongoc-counters-private.h"
 #include "mongoc/mongoc-error.h"
+#include "mongoc/mongoc-error-private.h"
 #include "mongoc/mongoc-log.h"
 #include "mongoc/mongoc-trace-private.h"
 #include "mongoc/mongoc-read-concern-private.h"
 #include "mongoc/mongoc-util-private.h"
 #include "mongoc/mongoc-write-concern-private.h"
 #include "mongoc/mongoc-read-prefs-private.h"
-
+#include "mongoc/mongoc-aggregate-private.h"
 
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "cursor"
@@ -248,6 +249,7 @@ _mongoc_cursor_new_with_opts (mongoc_client_t *client,
    cursor = (mongoc_cursor_t *) bson_malloc0 (sizeof *cursor);
    cursor->client = client;
    cursor->state = UNPRIMED;
+   cursor->client_generation = client->generation;
 
    bson_init (&cursor->opts);
    bson_init (&cursor->error_doc);
@@ -596,23 +598,26 @@ mongoc_cursor_destroy (mongoc_cursor_t *cursor)
       cursor->impl.destroy (&cursor->impl);
    }
 
-   if (cursor->in_exhaust) {
-      cursor->client->in_exhaust = false;
-      if (cursor->state != DONE) {
-         /* The only way to stop an exhaust cursor is to kill the connection */
-         mongoc_cluster_disconnect_node (
-            &cursor->client->cluster, cursor->server_id, false, NULL);
-      }
-   } else if (cursor->cursor_id) {
-      bson_strncpy (db, cursor->ns, cursor->dblen + 1);
+   if (cursor->client_generation == cursor->client->generation) {
+      if (cursor->in_exhaust) {
+         cursor->client->in_exhaust = false;
+         if (cursor->state != DONE) {
+            /* The only way to stop an exhaust cursor is to kill the connection
+             */
+            mongoc_cluster_disconnect_node (
+               &cursor->client->cluster, cursor->server_id, false, NULL);
+         }
+      } else if (cursor->cursor_id) {
+         bson_strncpy (db, cursor->ns, cursor->dblen + 1);
 
-      _mongoc_client_kill_cursor (cursor->client,
-                                  cursor->server_id,
-                                  cursor->cursor_id,
-                                  cursor->operation_id,
-                                  db,
-                                  cursor->ns + cursor->dblen + 1,
-                                  cursor->client_session);
+         _mongoc_client_kill_cursor (cursor->client,
+                                     cursor->server_id,
+                                     cursor->cursor_id,
+                                     cursor->operation_id,
+                                     db,
+                                     cursor->ns + cursor->dblen + 1,
+                                     cursor->client_session);
+      }
    }
 
    if (cursor->client_session && !cursor->explicit_session) {
@@ -901,9 +906,9 @@ bool
 _mongoc_cursor_run_command (mongoc_cursor_t *cursor,
                             const bson_t *command,
                             const bson_t *opts,
-                            bson_t *reply)
+                            bson_t *reply,
+                            bool retry_prohibited)
 {
-   mongoc_cluster_t *cluster;
    mongoc_server_stream_t *server_stream;
    bson_iter_t iter;
    mongoc_cmd_parts_t parts;
@@ -913,10 +918,10 @@ _mongoc_cursor_run_command (mongoc_cursor_t *cursor,
    char db[MONGOC_NAMESPACE_MAX];
    mongoc_session_opt_t *session_opts;
    bool ret = false;
+   bool is_retryable = true;
 
    ENTRY;
 
-   cluster = &cursor->client->cluster;
    mongoc_cmd_parts_init (
       &parts, cursor->client, db, MONGOC_QUERY_NONE, command);
    parts.is_read_command = true;
@@ -1005,6 +1010,24 @@ _mongoc_cursor_run_command (mongoc_cursor_t *cursor,
       parts.read_prefs = cursor->read_prefs;
    }
 
+   is_retryable = _is_retryable_read (&parts, server_stream);
+   if (!strcmp (cmd_name, "getMore")) {
+      is_retryable = false;
+   }
+   if (!strcmp (cmd_name, "aggregate")) {
+      bson_iter_t pipeline_iter;
+      if (bson_iter_init_find (&pipeline_iter, command, "pipeline") &&
+          BSON_ITER_HOLDS_ARRAY (&pipeline_iter) &&
+          bson_iter_recurse (&pipeline_iter, &pipeline_iter)) {
+         if (_has_write_key (&pipeline_iter)) {
+            is_retryable = false;
+         }
+      }
+   }
+   if (is_retryable && retry_prohibited) {
+      is_retryable = false;
+   }
+
    if (cursor->write_concern &&
        !mongoc_write_concern_is_default (cursor->write_concern) &&
        server_stream->sd->max_wire_version >= WIRE_VERSION_CMD_WRITE_CONCERN) {
@@ -1018,8 +1041,35 @@ _mongoc_cursor_run_command (mongoc_cursor_t *cursor,
       GOTO (done);
    }
 
+retry:
    ret = mongoc_cluster_run_command_monitored (
-      cluster, &parts.assembled, reply, &cursor->error);
+      &cursor->client->cluster, &parts.assembled, reply, &cursor->error);
+
+   if (ret) {
+      memset (&cursor->error, 0, sizeof (bson_error_t));
+   }
+
+   if (is_retryable &&
+       _mongoc_read_error_get_type (ret, &cursor->error, reply) ==
+          MONGOC_READ_ERR_RETRY) {
+      is_retryable = false;
+
+      mongoc_server_stream_cleanup (server_stream);
+
+      server_stream = mongoc_cluster_stream_for_reads (&cursor->client->cluster,
+                                                       cursor->read_prefs,
+                                                       cursor->client_session,
+                                                       reply,
+                                                       &cursor->error);
+
+      if (server_stream &&
+          server_stream->sd->max_wire_version >= WIRE_VERSION_RETRY_READS) {
+         cursor->server_id = server_stream->sd->id;
+         parts.assembled.server_stream = server_stream;
+         bson_destroy (reply);
+         GOTO (retry);
+      }
+   }
 
    if (cursor->error.domain) {
       bson_destroy (&cursor->error_doc);
@@ -1164,6 +1214,14 @@ mongoc_cursor_next (mongoc_cursor_t *cursor, const bson_t **bson)
 
    TRACE ("cursor_id(%" PRId64 ")", cursor->cursor_id);
 
+   if (cursor->client_generation != cursor->client->generation) {
+      bson_set_error (&cursor->error,
+                      MONGOC_ERROR_CURSOR,
+                      MONGOC_ERROR_CURSOR_INVALID_CURSOR,
+                      "Cannot advance cursor after client reset");
+      RETURN (false);
+   }
+
    if (bson) {
       *bson = NULL;
    }
@@ -1604,7 +1662,8 @@ _mongoc_cursor_response_refresh (mongoc_cursor_t *cursor,
 
    /* server replies to find / aggregate with {cursor: {id: N, firstBatch: []}},
     * to getMore command with {cursor: {id: N, nextBatch: []}}. */
-   if (_mongoc_cursor_run_command (cursor, command, opts, &response->reply) &&
+   if (_mongoc_cursor_run_command (
+          cursor, command, opts, &response->reply, false) &&
        _mongoc_cursor_start_reading_response (cursor, response)) {
       return;
    }
@@ -1617,7 +1676,6 @@ _mongoc_cursor_response_refresh (mongoc_cursor_t *cursor,
    }
 }
 
-
 void
 _mongoc_cursor_prepare_getmore_command (mongoc_cursor_t *cursor,
                                         bson_t *command)
