@@ -14,13 +14,11 @@
  * limitations under the License.
  */
 
+#include "mongoc/mongoc-prelude.h"
+
 #ifndef MONGOC_SOCKET_PRIVATE_H
 #define MONGOC_SOCKET_PRIVATE_H
 
-#if !defined(MONGOC_INSIDE) && !defined(MONGOC_COMPILATION)
-#error "Only <mongoc/mongoc.h> can be included directly."
-#endif
-
 #include "mongoc/mongoc-socket.h"
 
 BSON_BEGIN_DECLS
