@@ -14,13 +14,11 @@
  * limitations under the License.
  */
 
+#include "mongoc/mongoc-prelude.h"
+
 #ifndef MONGOC_URI_PRIVATE_H
 #define MONGOC_URI_PRIVATE_H
 
-#if !defined(MONGOC_COMPILATION)
-#error "Only <mongoc/mongoc.h> can be included directly."
-#endif
-
 #include "mongoc/mongoc-uri.h"
 
 
@@ -28,10 +26,17 @@ BSON_BEGIN_DECLS
 
 
 bool
-mongoc_uri_append_host (mongoc_uri_t *uri,
+mongoc_uri_upsert_host_and_port (mongoc_uri_t *uri,
+                                 const char *host_and_port,
+                                 bson_error_t *error);
+bool
+mongoc_uri_upsert_host (mongoc_uri_t *uri,
                         const char *host,
                         uint16_t port,
                         bson_error_t *error);
+void
+mongoc_uri_remove_host (mongoc_uri_t *uri, const char *host, uint16_t port);
+
 bool
 mongoc_uri_parse_host (mongoc_uri_t *uri, const char *str);
 bool
@@ -45,6 +50,13 @@ mongoc_uri_get_local_threshold_option (const mongoc_uri_t *uri);
 bool
 _mongoc_uri_requires_auth_negotiation (const mongoc_uri_t *uri);
 
+const char *
+mongoc_uri_canonicalize_option (const char *key);
+
+mongoc_uri_t *
+_mongoc_uri_copy_and_replace_host_list (const mongoc_uri_t *original,
+                                        const char *host);
+
 BSON_END_DECLS
 
 
