@@ -14,10 +14,7 @@
  * limitations under the License.
  */
 
-
-#if !defined(MONGOC_INSIDE) && !defined(MONGOC_COMPILATION)
-#error "Only <mongoc/mongoc.h> can be included directly."
-#endif
+#include "mongoc/mongoc-prelude.h"
 
 
 #ifndef MONGOC_VERSION_FUNCTIONS_H
