@@ -1,9 +1,14 @@
 <?php
 
+use MongoDB\Driver\BulkWrite;
 use MongoDB\Driver\Command;
 use MongoDB\Driver\Manager;
 use MongoDB\Driver\ReadPreference;
 use MongoDB\Driver\Server;
+use MongoDB\Driver\WriteConcern;
+use MongoDB\Driver\WriteConcernError;
+use MongoDB\Driver\WriteError;
+use MongoDB\Driver\WriteResult;
 use MongoDB\Driver\Exception\ConnectionException;
 use MongoDB\Driver\Exception\RuntimeException;
 
@@ -53,7 +58,7 @@ function drop_collection($uri, $databaseName, $collectionName)
         $server->executeCommand(
             $databaseName,
             $command,
-            [ 'writeConcern' => new \MongoDB\Driver\WriteConcern( \MongoDB\Driver\WriteConcern::MAJORITY ) ]
+            ['writeConcern' => new WriteConcern(WriteConcern::MAJORITY)]
         );
     } catch (RuntimeException $e) {
         if ($e->getMessage() !== 'ns not found') {
@@ -95,6 +100,18 @@ function get_primary_server($uri)
     return (new Manager($uri))->selectServer(new ReadPreference('primary'));
 }
 
+/**
+ * Returns a secondary server.
+ *
+ * @param string $uri Connection string
+ * @return Server
+ * @throws ConnectionException
+ */
+function get_secondary_server($uri)
+{
+    return (new Manager($uri))->selectServer(new ReadPreference('secondary'));
+}
+
 /**
  * Runs a command and returns whether an exception was thrown or not
  *
@@ -150,11 +167,11 @@ function get_server_storage_engine($uri)
 /**
  * Helper to return the version of a specific server.
  *
- * @param MongoDB\Driver\Server $server
+ * @param Server $server
  * @return string
  * @throws RuntimeException
  */
-function get_server_version_from_server(MongoDB\Driver\Server $server)
+function get_server_version_from_server(Server $server)
 {
     $command = new Command(['buildInfo' => 1]);
     $cursor = $server->executeCommand('admin', $command);
@@ -203,6 +220,35 @@ function is_mongos($uri)
     return get_primary_server($uri)->getType() === Server::TYPE_MONGOS;
 }
 
+/**
+ * Checks that the topology is a sharded cluster using a replica set
+ */
+function is_mongos_with_replica_set($uri)
+{
+    if (! is_mongos($uri)) {
+        return false;
+    }
+
+    $cursor = get_primary_server($uri)->executeQuery(
+        'config.shards',
+        new \MongoDB\Driver\Query([], ['limit' => 1])
+    );
+
+    $cursor->setTypeMap(['root' => 'array', 'document' => 'array']);
+    $document = current($cursor->toArray());
+
+    if (! $document) {
+        return false;
+    }
+
+    /**
+     * Use regular expression to distinguish between standalone or replicaset:
+     * Without a replicaset: "host" : "localhost:4100"
+     * With a replicaset: "host" : "dec6d8a7-9bc1-4c0e-960c-615f860b956f/localhost:4400,localhost:4401"
+     */
+    return preg_match('@^.*/.*:\d+@', $document['host']);
+}
+
 /**
  * Checks that the topology is a replica set.
  *
@@ -249,7 +295,7 @@ function is_auth($uri)
  */
 function is_ssl($uri)
 {
-    return stripos($uri, 'ssl=true') !== false;
+    return stripos($uri, 'ssl=true') !== false || stripos($uri, 'tls=true') !== false;
 }
 
 /**
@@ -414,30 +460,21 @@ function NEEDS($configuration) {
         exit("skip -- need '$configuration' defined");
     }
 }
-function PREDICTABLE() {
-    global $servers;
-
-    foreach($servers as $k => $v) {
-        if (!defined($k) || !constant($k)) {
-            exit("skip - needs predictable environment (e.g. vagrant)\n");
-        }
-    }
-}
 function SLOW() {
     if (getenv("SKIP_SLOW_TESTS")) {
         exit("skip SKIP_SLOW_TESTS");
     }
 }
 
-function loadFixtures(\MongoDB\Driver\Manager $manager, $dbname = DATABASE_NAME, $collname = COLLECTION_NAME, $filename = null)
+function loadFixtures(Manager $manager, $dbname = DATABASE_NAME, $collname = COLLECTION_NAME, $filename = null)
 {
     if (!$filename) {
         $filename = "compress.zlib://" . __DIR__ . "/" . "PHONGO-FIXTURES.json.gz";
     }
 
-    $bulk = new MongoDB\Driver\BulkWrite(['ordered' => false]);
+    $bulk = new BulkWrite(['ordered' => false]);
 
-    $server = $manager->selectServer(new MongoDB\Driver\ReadPreference(MongoDB\Driver\ReadPreference::RP_PRIMARY));
+    $server = $manager->selectServer(new ReadPreference(ReadPreference::RP_PRIMARY));
 
     $data = file_get_contents($filename);
     $array = json_decode($data);
@@ -460,8 +497,7 @@ function createTemporaryMongoInstance(array $options = [])
         "name" => "mongod",
         "id" => $id,
         'procParams' => [
-            'dbpath' => "/tmp/phongo/{$id}",
-            'logpath' => "/tmp/phongo/{$id}.log",
+            'logpath' => "/tmp/MO/phongo/{$id}.log",
             'ipv6' => true,
             'setParameter' => [ 'enableTestCommands' => 1 ],
         ],
@@ -509,8 +545,12 @@ function destroyTemporaryMongoInstance($id = NULL)
 
 function severityToString($type) {
     switch($type) {
+    case E_RECOVERABLE_ERROR:
+        return "E_RECOVERABLE_ERROR";
     case E_WARNING:
         return "E_WARNING";
+    case E_NOTICE:
+        return "E_NOTICE";
     default:
         return "Some other #_$type";
     }
@@ -524,6 +564,8 @@ function raises($function, $type, $infunction = null) {
     try {
         $function();
     } catch(Exception $e) {
+        $exceptionname = get_class($e);
+
         if ($e instanceof ErrorException && $e->getSeverity() & $type) {
             if ($infunction) {
                 $trace = $e->getTrace();
@@ -544,39 +586,47 @@ function raises($function, $type, $infunction = null) {
         return $e->getMessage();
     }
 
-    echo "FAILED: Expected $exceptionname thrown!\n";
+    printf("FAILED: Expected %s thrown!\n", ErrorException::class);
     restore_error_handler();
 }
 function throws($function, $exceptionname, $infunction = null) {
     try {
         $function();
+    } catch (Throwable $e) {
     } catch(Exception $e) {
-        $message = str_replace(array("\n", "\r"), ' ', $e->getMessage());
-        if ($e instanceof $exceptionname) {
-            if ($infunction) {
-                $trace = $e->getTrace();
-                $function = $trace[0]["function"];
-                if (strcasecmp($function, $infunction) == 0) {
-                    printf("OK: Got %s thrown from %s\n", $exceptionname, $infunction);
-                } else {
-                    printf("ALMOST: Got %s - but was thrown in %s, not %s (%s)\n", $exceptionname, $function, $infunction, $message);
-                }
-                return $e->getMessage();
-            }
-            printf("OK: Got %s\n", $exceptionname);
-        } else {
-            printf("ALMOST: Got %s (%s) - expected %s\n", get_class($e), $message, $exceptionname);
-        }
-        return $e->getMessage();
     }
-    echo "FAILED: Expected $exceptionname thrown, but no exception thrown!\n";
+
+    if ($e === null) {
+        echo "FAILED: Expected $exceptionname thrown, but no exception thrown!\n";
+        return;
+    }
+
+    $message = str_replace(array("\n", "\r"), ' ', $e->getMessage());
+    if ($e instanceof $exceptionname) {
+        if ($infunction) {
+            $trace = $e->getTrace();
+            $function = $trace[0]["function"];
+            if (strcasecmp($function, $infunction) == 0) {
+                printf("OK: Got %s thrown from %s\n", $exceptionname, $infunction);
+            } else {
+                printf("ALMOST: Got %s - but was thrown in %s, not %s (%s)\n", $exceptionname, $function, $infunction, $message);
+            }
+            return $e->getMessage();
+        }
+        printf("OK: Got %s\n", $exceptionname);
+    } else {
+        printf("ALMOST: Got %s (%s) - expected %s\n", get_class($e), $message, $exceptionname);
+    }
+
+    return $e->getMessage();
 }
-function printServer(MongoDB\Driver\Server $server)
+
+function printServer(Server $server)
 {
     printf("server: %s:%d\n", $server->getHost(), $server->getPort());
 }
 
-function printWriteResult(MongoDB\Driver\WriteResult $result, $details = true)
+function printWriteResult(WriteResult $result, $details = true)
 {
     printServer($result->getServer());
 
@@ -599,7 +649,7 @@ function printWriteResult(MongoDB\Driver\WriteResult $result, $details = true)
     }
 }
 
-function printWriteConcernError(MongoDB\Driver\WriteConcernError $error = null, $details)
+function printWriteConcernError(WriteConcernError $error = null, $details)
 {
     if ($error) {
         /* This stuff is generated by the server, no need for us to test it */
@@ -615,7 +665,7 @@ function printWriteConcernError(MongoDB\Driver\WriteConcernError $error = null,
     }
 }
 
-function printWriteError(MongoDB\Driver\WriteError $error)
+function printWriteError(WriteError $error)
 {
     var_dump($error);
     printf("writeError[%d].message: %s\n", $error->getIndex(), $error->getMessage());
@@ -644,7 +694,21 @@ function def($arr) {
     }
 }
 
-function configureFailPoint(MongoDB\Driver\Manager $manager, $failPoint, $mode, array $data = [])
+function configureFailPoint(Manager $manager, $failPoint, $mode, array $data = [])
+{
+    $doc = [
+        'configureFailPoint' => $failPoint,
+        'mode'               => $mode,
+    ];
+    if ($data) {
+        $doc['data'] = $data;
+    }
+
+    $cmd = new Command($doc);
+    $manager->executeCommand('admin', $cmd);
+}
+
+function configureTargetedFailPoint(Server $server, $failPoint, $mode, array $data = [])
 {
     $doc = array(
         'configureFailPoint' => $failPoint,
@@ -654,13 +718,13 @@ function configureFailPoint(MongoDB\Driver\Manager $manager, $failPoint, $mode,
         $doc['data'] = $data;
     }
 
-    $cmd = new MongoDB\Driver\Command($doc);
-    $manager->executeCommand('admin', $cmd);
+    $cmd = new Command($doc);
+    $server->executeCommand('admin', $cmd);
 }
 
-function failMaxTimeMS(MongoDB\Driver\Manager $manager)
+function failMaxTimeMS(Server $server)
 {
-    configureFailPoint($manager, 'maxTimeAlwaysTimeOut', [ 'times' => 1 ]);
+    configureTargetedFailPoint($server, 'maxTimeAlwaysTimeOut', [ 'times' => 1 ]);
 }
 
 function getMOPresetBase() {
@@ -692,7 +756,7 @@ function fromJSON($var) {
 
 /* Note: this fail point may terminate the mongod process, so you may want to
  * use this in conjunction with a throwaway server. */
-function failGetMore(MongoDB\Driver\Manager $manager)
+function failGetMore(Manager $manager)
 {
     /* We need to do version detection here */
     $primary = $manager->selectServer(new ReadPreference('primary'));
